package com.sf.service.gateway.domain.dto;

import com.sf.common.annotation.Excel;
import com.sf.file.domain.SysOss;
import com.sf.service.gateway.domain.GatewayInterfaceInfo;
import com.sf.service.gateway.domain.GatewayInterfaceMock;
import com.sf.service.gateway.domain.GatewayServer;
import com.sf.service.gateway.domain.vo.GatewayInterfaceInfoListVO;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.*;
import java.util.List;

import static org.apache.commons.lang3.StringUtils.EMPTY;


/**
 * 新增或修改接口信息对象 Gateway_interface_info
 * 
 * @author zoukun
 * @date 2024-04-22
 */
@Data
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "InsertOrUpdateGatewayInterfaceInfoDTO", description = "新增或修改接口信息对象")
public class InsertOrUpdateGatewayInterfaceInfoDTO {
    private static final long serialVersionUID = 1L;

    @ApiModelProperty("ID")
    private Long id;

    @Length(max = 50,message = "接口名称不能超过50个字符")
    @NotBlank(message = "接口名称不能为空")
    @ApiModelProperty("接口名称")
    private String interfaceName;

    /** api编码 */
    @Excel(name = "接口编码")
    @NotBlank(message = "接口编码不能为空")
    @Pattern(regexp = "^[a-zA-Z][a-zA-Z0-9.-]{0,63}$",message = "接口编码只能以字母开头，只能包含字母、数字和.-，不超过64个字符")
    private String apiCode;

    @Length(max = 200,message = "接口名称不能超过200个字符")
    @NotBlank(message = "接口路径不能为空")
    @ApiModelProperty("接口路径")
    private String interfacePath;

    @NotBlank(message = "请求方式不能为空")
    @ApiModelProperty("请求方式")
    private String requestMethod;

    @ApiModelProperty("接口描述")
    private String description;

    @Length(max = 20,message = "接口版本不能超过20个字符")
    @NotBlank(message = "接口版本不能为空")
    @ApiModelProperty("接口版本")
    private String version;

    @ApiModelProperty("接口文档（文档地址）")
    private SysOss document;

    @NotNull(message = "网关服务不能为空")
    @ApiModelProperty("网关服务id")
    private Long serverId;

    @DecimalMin(value = "1",message = "超时时间为大于等于1的正整数")
    @DecimalMax(value = "150000",message = "超时时间不能大于150000")
    @ApiModelProperty("超时时间，单位毫秒,1-150000之间")
    private Integer timeout;

    @ApiModelProperty("策略ID集合")
    private List<Long> strategyIds;

    @ApiModelProperty("Mock状态(0=停用,1=启用)")
    private String mockStatus;

    @ApiModelProperty("Mock默认响应HTTP状态码")
    private Integer mockDefaultHttpStatus;

    @ApiModelProperty("Mock默认响应，json格式")
    private String mockDefaultResponse;

    @ApiModelProperty("Mock默认响应，json格式")
    private List<InsertGatewayInterfaceMockDTO> mockExpectations;

    public GatewayInterfaceInfo convertToGatewayInterfaceInfo() {
        GatewayInterfaceInfo gatewayInterfaceInfo = new GatewayInterfaceInfo();
        gatewayInterfaceInfo.setId(this.id);
        gatewayInterfaceInfo.setInterfaceName(this.interfaceName);
        gatewayInterfaceInfo.setApiCode(this.apiCode);
        gatewayInterfaceInfo.setInterfacePath(this.interfacePath);
        gatewayInterfaceInfo.setRequestMethod(this.requestMethod);
        gatewayInterfaceInfo.setDescription(this.description);
        gatewayInterfaceInfo.setVersion(this.version);
        String document = this.document == null ? EMPTY : this.document.getId();
        gatewayInterfaceInfo.setDocument(document);
        gatewayInterfaceInfo.setServerId(this.serverId);
        gatewayInterfaceInfo.setTimeout(this.timeout);
        gatewayInterfaceInfo.setMockStatus(this.mockStatus);
        gatewayInterfaceInfo.setMockDefaultHttpStatus(this.mockDefaultHttpStatus);
        gatewayInterfaceInfo.setMockDefaultResponse(this.mockDefaultResponse);
        return gatewayInterfaceInfo;
    }

}
