package com.sf.service.gateway.domain.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.sf.service.gateway.domain.GatewayRoute;
import com.sf.service.gateway.domain.GatewayServer;
import com.sf.vertx.enums.GatewayServiceModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.Date;

/**
 * 服务列表
 *
 * @author zoukun
 * @date 2024-04-22
 */
@Data
public class GatewayServerListVO {
    private static final long serialVersionUID = 1L;

    private Long id;

    @ApiModelProperty("服务名称")
    private String serverName;

    @ApiModelProperty("服务地址，路由模式为路由名称")
    private String serverAddress;

    @ApiModelProperty("备注")
    private String remark;

    @ApiModelProperty("服务状态")
    private String status;

    @ApiModelProperty("创建者")
    private String created;

    @ApiModelProperty("更新者")
    private String modified;

    @ApiModelProperty("服务模式（NORMAL=普通模式，ROUTE=路由模式）")
    private String serviceModel;

    @ApiModelProperty("路由规则id")
    private Long routeId;

    @ApiModelProperty(" 服务类型，SAC=SAC规范服务，OPEN=开放服务")
    private String serviceType;

    @ApiModelProperty("超时时间，单位毫秒,1-150000之间")
    private Integer timeout;

    /**
     * 创建时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    /**
     * 更新时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;


    public static GatewayServerListVO convert(GatewayServer gatewayServer, GatewayRoute linkRoute) {
        if (gatewayServer == null) {
            return null;
        }
        GatewayServerListVO vo = new GatewayServerListVO();

        vo.setId(gatewayServer.getId());
        vo.setServerName(gatewayServer.getServerName());
        vo.setServerAddress(GatewayServiceModel.ROUTE.getCode().equals(gatewayServer.getServiceModel()) ?
                linkRoute.getRouteName()
                : gatewayServer.getServerAddress());
        vo.setRemark(gatewayServer.getRemark());
        vo.setStatus(gatewayServer.getStatus());
        vo.setCreated(gatewayServer.getCreated());
        vo.setModified(gatewayServer.getModified());
        vo.setServiceModel(gatewayServer.getServiceModel());
        vo.setServiceType(gatewayServer.getServiceType());
        vo.setRouteId(gatewayServer.getRouteId());
        vo.setTimeout(gatewayServer.getTimeout());
        vo.setCreateTime(gatewayServer.getCreateTime());
        vo.setUpdateTime(gatewayServer.getUpdateTime());
        return vo;
    }
}
