/*
package com.sf.vertx.handle;

import cn.hutool.core.util.StrUtil;
import com.sf.vertx.api.pojo.ApiConfig;
import com.sf.vertx.api.pojo.AppConfig;
import com.sf.vertx.enums.GatewayError;
import com.sf.vertx.exception.ServiceException;
import com.sf.vertx.utils.AppUtils;
import io.vertx.ext.web.RoutingContext;
import lombok.extern.slf4j.Slf4j;

import static com.sf.vertx.constans.SACConstants.*;

@Slf4j
public class SACParameterCheckHandlerImpl implements ParameterCheckHandler {

    @Override
    public void handle(RoutingContext rc) {
        try {
            log.info("Enter SAC Route");
            // SAC规范要求Header必须要同时包含appCode和apiCode
            String appCode = rc.request().headers().get(AppConfigHandler.getAppCodeHeaderKey());
            String apiCode = rc.request().headers().get(AppConfigHandler.getApiCodeHeaderKey());
            if (StrUtil.isBlank(appCode) || StrUtil.isBlank(apiCode)) {
                rc.fail(new ServiceException(GatewayError.PARAMETER_TRANSFER_ERROR));
                return;
            }
            AppConfig appConfig = AppConfigHandler.getAppConfig(appCode);
            if (appConfig == null) {
                rc.fail(new ServiceException(GatewayError.APP_SERVICE_NOT_FOUND));
                return;
            }
            String apiCacheKey = appConfig.getAppCode() + CACHE_KEY_CONNECTOR + apiCode;
            ApiConfig apiConfig = AppConfigHandler.getApicodeConfig(apiCacheKey);
            if (apiConfig == null) {
                rc.fail(new ServiceException(GatewayError.API_SERVICE_NOT_FOUND));
                return;
            }
            // 设置应用配置、接口配置到上下文
            AppUtils.setAppConfigToRoutingContext(appConfig,rc);
            AppUtils.setApiConfigIntoRoutingContext(apiConfig,rc);
            rc.put(API_SERVICE_TYPE,AppConfigHandler.getServiceTypeOpen(apiCacheKey));
        } catch (Exception e) {
            log.error("SACParameterCheckHandlerImpl Error:",e);
            rc.fail(new ServiceException(GatewayError.DEFAULT_SERVICE_ERROR));
            return;
        }
        rc.next();
    }
}
*/
